function nn = nnbp(nn)
%NNBP performs backpropagation
% nn = nnbp(nn) returns an neural network structure with updated weights 
    
    n = nn.n;
    sparsityError = 0;

    [mm,mmm] = size(nn.a{2});
    [ww,www] = size(nn.W{2});

    switch nn.output
        case 'sigm'
            d{n} = - nn.e .* (nn.a{n} .* (1 - nn.a{n}));
        case {'softmax','linear'}
            d{n} = - nn.e;
    end
    for i = (n - 1) : -1 : 2
        % Derivative of the activation function
        switch nn.activation_function 
            case 'sigm'
                d_act = nn.a{i} .* (1 - nn.a{i});
            case 'tanh_opt'
                d_act = 1.7159 * 2/3 * (1 - 1/(1.7159)^2 * nn.a{i}.^2);
        end
        
        if(nn.nonSparsityPenalty>0)
            pi = repmat(nn.p{i}, size(nn.a{i}, 1), 1);
            sparsityError = [zeros(size(nn.a{i},1),1) nn.nonSparsityPenalty * (-nn.sparsityTarget ./ pi + (1 - nn.sparsityTarget) ./ (1 - pi))];
        end

        %sparsityError = 0;
        
        % Backpropagate first derivatives
        if i+1==n % in this case in d{n} there is not the bias term to be removed             
            d{i} = (d{i + 1} * nn.W{i}) .* d_act; %( Bishop (5.56)
        else % in this case in d{i} the bias term has to be removed
            if i==2
                da2 = nn.p{i} .* (1 - nn.p{i});
                da1 = nn.a{i} .* (1 - nn.a{i});
              
                dg = diag(nn.W{i}(:,2:end));
                og = nn.W{i}(:,2:end);
                [gg1,gg2] = size(og);
                for kg = 1:gg1
                    og(kg,kg) = 1;
                end

                gg = sum(og, 2);
                ggg = gg.*dg;

                pre = d{i + 2} * nn.W{i+1};
                %ddd = (d{i + 1}(:,2:end) * daa + sparsityError) .* da2;
                %ddd1 = (d{i + 1}(:,2:end) .* ggg' + sparsityError);

                %ddd1 = (d{i + 1}(:,2:end) .* dg' + sparsityError);
                wk = nn.W{i}(:,2:end);
                [wk1,wk2] = size(wk);

                for k1 = 1:wk1
                    for k2 = 1:wk2
                        if k1 == k2
                            wk(k1,k2) = 1;
                        end
                    end
                end
 
                ddd = ((d{i + 1}(:,2:end) * wk + sparsityError) .* ((da1(:,2:end) * (n-1) + ones(1,wk2)) / wk1) * diag(diag(nn.W{i}(:,2:end))) ) .* da2(:,2:end);              
                %ddd2 = [zeros(mm,1) ddd1];
                %d{i} = ddd2.* da2;
                d{i} = [zeros(mm,1) ddd];
            end

        end
        
        if(nn.dropoutFraction>0)
            d{i} = d{i} .* [ones(size(d{i},1),1) nn.dropOutMask{i}];
        end

    end

    for i = 1 : (n - 1)
        if i+1==n
            nn.dW{i} = (d{i + 1}' * nn.a{i}) / size(d{i + 1}, 1);
        else
            if i== 1
                nn.dW{i} = (d{i + 1}(:,2:end)' * nn.a{i}) / size(d{i + 1}, 1); 
            else 
                if i == 2
                    %for j = 1:50
                    %    D = d{i + 1}(:,j+1)';
                    %    A = nn.a{i};
    
                        %D(:,j) = [];
                    %    A(:,j) = zeros(100,1);seros
        
                    %    nn.dW{i}(j,:) = D * A;
                    %end
                    %nn.dW{i} = nn.dW{i} / size(d{i + 1}, 1);
                    disp("bp");
                    disp(i);
    
                    %for j = 1:ww
                        for k = 2:www
                            ap = nn.a{i};
                            ap(:,k) = nn.p{i}(:,k);

                            dw1 = (d{i + 1}(:,k)' * ap) / size(d{i + 1}, 1);

                            if k == 2
                                ddw = dw1;
                            else
                                ddw = [ddw;dw1];
                            end
                        end
                    %end

                    dw = ddw;

                    
    
                    %[dw1,dw2] = size(dw);
    
                    %nn.dW{i} = zeros(dw1,dw2);
                    nn.dW{i} = dw;

                    nn.dW{i} = nn.dW{i} + nn.weightPenaltyL1 * sign(nn.W{i});
                end
            end
        end
    end
end
